@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #ifndef GOG_GURU_H
@@ -32,8 +32,9 @@ G_BEGIN_DECLS
 /* typedef void (*GogGuruRegister) (GogGraph *graph, gpointer user); */
 
 GtkWidget *gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
-		     GOCmdContext *cc, GtkWindow *toplevel,
-		     GClosure *closure);
+		     GOCmdContext *cc, GClosure *closure);
+
+GtkWidget *gog_guru_get_help_button (GtkWidget *guru);
 
 G_END_DECLS
 
