@@ -25,8 +25,7 @@
 #include "goffice-gtk.h"
 #include <goffice/utils/go-glib-extras.h>
 
-#include <gtk/gtkaction.h>
-#include <gtk/gtktoolitem.h>
+#include <gtk/gtk.h>
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 
@@ -53,6 +52,7 @@ go_tool_combo_text_create_menu_proxy (GtkToolItem *tool_item)
 }
 #endif
 
+#ifndef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
 static gboolean
 go_tool_combo_text_set_tooltip (GtkToolItem *tool_item, GtkTooltips *tooltips,
 				char const *tip_text,
@@ -63,6 +63,7 @@ go_tool_combo_text_set_tooltip (GtkToolItem *tool_item, GtkTooltips *tooltips,
 				  tip_text, tip_private);
 	return TRUE;
 }
+#endif
 
 static void
 go_tool_combo_text_class_init (GtkToolItemClass *tool_item_klass)
@@ -71,7 +72,9 @@ go_tool_combo_text_class_init (GtkToolItemClass *tool_item_klass)
 	gobject_klass->finalize		   = go_tool_combo_text_finalize;
 	tool_item_klass->create_menu_proxy = go_tool_combo_text_create_menu_proxy;
 #endif
+#ifndef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
 	tool_item_klass->set_tooltip	   = go_tool_combo_text_set_tooltip;
+#endif
 }
 
 static GSF_CLASS (GOToolComboText, go_tool_combo_text,
@@ -107,19 +110,6 @@ set_entry_val (GOActionComboText *taction, char const *text)
 	}
 }
 
-#if 0
-static void
-go_action_combo_text_connect_proxy (GtkAction *action, GtkWidget *proxy)
-{
-}
-
-static void
-go_action_combo_disconnect_proxy (GtkAction *action,
-				  GtkWidget *proxy)
-{
-}
-#endif
-
 static gint
 g_strcase_equal (gconstpointer s1, gconstpointer s2)
 {
@@ -135,35 +125,29 @@ cb_entry_changed (GoComboText *ct, char const *text, GOActionComboText *taction)
 }
 
 static GtkWidget *
-go_action_combo_create_tool_item (GtkAction *act)
+go_action_combo_text_create_tool_item (GtkAction *act)
 {
 	GOActionComboText *taction = GO_ACTION_COMBO_TEXT (act);
 	GOToolComboText *tool = g_object_new (GO_TOOL_COMBO_TEXT_TYPE, NULL);
 	GSList *ptr;
-	int tmp, w = -1;
+	int w = -1;
 
 	tool->combo = (GoComboText *)go_combo_text_new (taction->case_sensitive ? NULL : g_strcase_equal);
 	if (taction->largest_elem != NULL)
-		w = go_pango_measure_string (
-			gtk_widget_get_pango_context (GTK_WIDGET (tool->combo)),
-			go_combo_text_get_entry (tool->combo)->style->font_desc, 
-			taction->largest_elem);
+		w = g_utf8_strlen (taction->largest_elem, -1);
+
 	for (ptr = taction->elements; ptr != NULL ; ptr = ptr->next) {
 		go_combo_text_add_item	(tool->combo, ptr->data);
 		if (taction->largest_elem == NULL) {
-			tmp = go_pango_measure_string (
-				gtk_widget_get_pango_context (GTK_WIDGET (tool->combo)),
-				go_combo_text_get_entry (tool->combo)->style->font_desc, 
-				ptr->data);
+			int tmp = g_utf8_strlen (ptr->data, -1);
 			if (w < tmp)
 				w = tmp;
 		}
 	}
-				
+
 	go_combo_box_set_title (GO_COMBO_BOX (tool->combo),
 		_(gtk_action_get_name (act)));
-	gtk_widget_set_size_request (
-		go_combo_text_get_entry (tool->combo), w, -1);
+	gtk_entry_set_width_chars (GTK_ENTRY (go_combo_text_get_entry (tool->combo)), w);
 	g_object_set (G_OBJECT (tool), "visible-vertical", FALSE, NULL);
 
 	go_combo_box_set_relief (GO_COMBO_BOX (tool->combo), GTK_RELIEF_NONE);
@@ -176,9 +160,43 @@ go_action_combo_create_tool_item (GtkAction *act)
 	return GTK_WIDGET (tool);
 }
 
+static void
+cb_menu_activated (GtkMenuItem *item, GOActionComboText *taction)
+{
+	const char *text = g_object_get_data (G_OBJECT (item), "text");
+	set_entry_val (taction, text);
+	gtk_action_activate (GTK_ACTION (taction));
+}
+
+static GtkWidget *
+go_action_combo_text_create_menu_item (GtkAction *act)
+{
+	GOActionComboText *taction = GO_ACTION_COMBO_TEXT (act);
+	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *item = gtk_image_menu_item_new ();
+	GSList *ptr;
+
+	for (ptr = taction->elements; ptr != NULL ; ptr = ptr->next) {
+		const char *text = ptr->data;
+		GtkWidget *item = gtk_menu_item_new_with_label (text);
+		g_object_set_data (G_OBJECT (item), "text", (gpointer)text);
+		g_signal_connect (item, "activate",
+				  G_CALLBACK (cb_menu_activated),
+				  act);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
+	gtk_widget_show_all (item);
+	return item;
+}
+
 static void
 go_action_combo_text_finalize (GObject *obj)
 {
+	GOActionComboText *taction = GO_ACTION_COMBO_TEXT (obj);
+	go_slist_free_custom (taction->elements, (GFreeFunc)g_free);
 	combo_text_parent->finalize (obj);
 }
 
@@ -201,9 +219,9 @@ go_action_combo_text_set_property (GObject      *object,
 
 static void
 go_action_combo_text_get_property (GObject      *object,
-		  guint         prop_id,
-		  GValue       *value,
-		  GParamSpec   *pspec)
+				   guint         prop_id,
+				   GValue       *value,
+				   GParamSpec   *pspec)
 {
 	GOActionComboText *taction = GO_ACTION_COMBO_TEXT (object);
 
@@ -224,12 +242,8 @@ go_action_combo_text_class_init (GtkActionClass *gtk_act_klass)
 	combo_text_parent = g_type_class_peek_parent (gobject_klass);
 	gobject_klass->finalize		= go_action_combo_text_finalize;
 
-	gtk_act_klass->create_tool_item = go_action_combo_create_tool_item;
-#if 0
-	gtk_act_klass->create_menu_item = Use the default
-	gtk_act_klass->connect_proxy	= go_action_combo_stack_connect_proxy;
-	gtk_act_klass->disconnect_proxy = go_action_combo_stack_disconnect_proxy;
-#endif
+	gtk_act_klass->create_tool_item = go_action_combo_text_create_tool_item;
+	gtk_act_klass->create_menu_item = go_action_combo_text_create_menu_item;
 
 	gobject_klass->set_property	= go_action_combo_text_set_property;
 	gobject_klass->get_property	= go_action_combo_text_get_property;
@@ -237,7 +251,7 @@ go_action_combo_text_class_init (GtkActionClass *gtk_act_klass)
 	g_object_class_install_property (gobject_klass,
 		PROP_CASE_SENSITIVE,
 		g_param_spec_boolean ("case-sensitive", _("Case Sensitive"),
-			_("Should the text comparasion be case sensitive"),
+			_("Should the text comparison be case sensitive"),
 			TRUE,
 			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
 			GSF_PARAM_STATIC));
